/*
    Game of Life v0.1
        by Nicolas Lopez Fernandez

    Description:
        This is a test program of Conway's Game of Life on MS-DOS.
        An equivalent program will be ported to a 18F4455 / 18F2455  PIC microcontroller.

    License:
        This work is licensed under the Creative Commons BY-NC-SA License.
        To view a copy of this license, visit http://creativecommons.org
*/

#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#define setbit(pos, var) (var | (1 << (pos)))
#define clearbit(pos, var) (var & ~(1 << (pos)))
#define getbit(pos, var) !!(var & (1 << (pos)))

//typedef short int byte;       //-> C
//typedef unsigned int word;    //-> C

typedef unsigned char byte;     // -> PC
typedef short int word;         // -> PC

void ChangeGame(word *previous, word *next);
void WriteGame(word *previous, word *next);
void DisplayGame(word *previous);

void wait(long milliseconds);

int main(int argc, char *argv[])
{
    byte i, j;
    word gameOfLifePrevious[16] = {0};
    word gameOfLifeNext[16] = {0};
    /*{  // Test
        0b1000001100000011, //  0
        0b1000000000000000, //  1
        0b0000000001000000, //  2
        0b0000000000000000, //  3
        0b1000000000000011, //  4
        0b0000000000000000, //  5
        0b0000000000000000, //  6
        0b0000000000000000, //  7
        0b0001111111111000, //  8
        0b0000000000000000, //  9
        0b0000000000000000, // 10
        0b1100000000000011, // 11
        0b0000000000000000, // 12
        0b0000000000000000, // 13
        0b0000100000000000, // 14
        0b1100100011000011, // 15
    };*/

    srand(time(NULL));

    for (i = 0; i < 16; i++)
    {
        for (j = 0; j < 16; j++)
        {
            if (rand()%2)
            {
                gameOfLifePrevious[i] = setbit(j, gameOfLifePrevious[i]);
                gameOfLifeNext[i] = setbit(j, gameOfLifeNext[i]);
            }
            else
            {
                gameOfLifePrevious[i] = clearbit(j, gameOfLifePrevious[i]);
                gameOfLifeNext[i] = clearbit(j, gameOfLifeNext[i]);
            }
        }
    }

    while (1)
    {
        WriteGame(gameOfLifePrevious, gameOfLifeNext);
        DisplayGame(gameOfLifePrevious);
        wait(1000);
        ChangeGame(gameOfLifePrevious, gameOfLifeNext);
        system("cls");
    }

    return 0;
}

void ChangeGame(word *previous, word *next)
{
    byte i, j;
    byte counter;

    for (i = 0; i < 16; i++)
    {
        for (j = 0; j < 16; j++)
        {
            if (i == 0)
            {
                if (j == 15)
                    counter = getbit(j, previous[i + 1]) + getbit(j - 1, previous[i + 1]) + getbit(j - 1, previous[i]);
                else if (j == 0)
                    counter = getbit(j, previous[i + 1]) + getbit(j + 1, previous[i + 1]) + getbit(j + 1, previous[i]);
                else
                    counter = getbit(j - 1, previous[i]) + getbit(j + 1, previous[i]) + getbit(j - 1, previous[i + 1]) + getbit(j, previous[i + 1]) + getbit(j + 1, previous[i + 1]);
            }
            else if (i == 15)
            {
                if (j == 15)
                    counter = getbit(j - 1, previous[i]) + getbit(j - 1, previous[i - 1]) + getbit(j, previous[i - 1]);
                else if (j == 0)
                    counter = getbit(j, previous[i - 1]) + getbit(j + 1, previous[i]) + getbit(j + 1, previous[i - 1]);
                else
                    counter = getbit(j - 1, previous[i]) + getbit(j + 1, previous[i]) + getbit(j - 1, previous[i - 1]) + getbit(j, previous[i - 1]) + getbit(j + 1, previous[i - 1]);
            }
            else if (j == 0)
                counter = getbit(j, previous[i - 1]) + getbit(j, previous[i + 1]) + getbit(j + 1, previous[i - 1]) + getbit(j + 1, previous[i]) + getbit(j + 1, previous[i + 1]);
            else if (j == 15)
                counter = getbit(j, previous[i - 1]) + getbit(j, previous[i + 1]) + getbit(j - 1, previous[i - 1]) + getbit(j - 1, previous[i]) + getbit(j - 1, previous[i + 1]);
            else if (i >= 1 && i <= 14 && j >= 1 && j <= 14)
            {
                counter = getbit(j - 1, previous[i - 1]) + getbit(j, previous[i - 1]) + getbit(j + 1, previous[i - 1]);
                counter += getbit(j - 1, previous[i]) + getbit(j + 1, previous[i]);
                counter += getbit(j - 1, previous[i + 1]) + getbit(j, previous[i + 1]) + getbit(j + 1, previous[i + 1]);
            }

            if (getbit(j, previous[i]))
            {
                if (counter != 2 && counter !=3)
                    next[i] = clearbit(j, next[i]);
            }
            else
            {
                if (counter == 3)
                    next[i] = setbit(j, next[i]);
            }

            counter = 0;
        }
    }
}

void WriteGame(word *previous, word *next)
{
    byte i;

    for (i = 0; i < 16; i++)
    {
        previous[i] = next[i];
    }
}

void DisplayGame(word *previous)
{
    byte i, j;

    for (i = 0; i < 16; i++)
    {
        for (j = 16; j > 0; j--)
        {
            printf("%c ", getbit(j-1, previous[i]) ? (char)254 : ' ');
        }

        printf("\n");
    }
}

void wait(long milliseconds)
{
    long timeout = clock() + milliseconds;
    while (clock() < timeout);
}
